@@ -519,7 +519,10 @@ NOEXPORT int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
     X509 *cert;
     OCSP_CERTID *cert_id;
     STACK_OF(OPENSSL_STRING) *aia;
-    int i, status=V_OCSP_CERTSTATUS_UNKNOWN;
+    int i, ocsp_status=V_OCSP_CERTSTATUS_UNKNOWN, saved_error;
+
+    /* the original error code is restored unless we report our own error */
+    saved_error=X509_STORE_CTX_get_error(callback_ctx);
 
     /* get the current certificate ID */
     cert=X509_STORE_CTX_get_current_cert(callback_ctx);
@@ -544,7 +547,7 @@ NOEXPORT int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
 
     /* use the responder specified in the configuration file */
     if(c->opt->ocsp_url) {
-        s_log(LOG_DEBUG, "OCSP: Connecting configured responder \"%s\"",
+        s_log(LOG_NOTICE, "OCSP: Connecting configured responder \"%s\"",
             c->opt->ocsp_url);
         if(ocsp_request(c, callback_ctx, cert_id, c->opt->ocsp_url)!=
                 V_OCSP_CERTSTATUS_GOOD) {
@@ -556,21 +559,22 @@ NOEXPORT int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
     /* use the responder from AIA (Authority Information Access) */
     if(c->opt->option.aia && (aia=X509_get1_ocsp(cert))) {
         for(i=0; i<sk_OPENSSL_STRING_num(aia); i++) {
-            s_log(LOG_DEBUG, "OCSP: Connecting AIA responder \"%s\"",
+            s_log(LOG_NOTICE, "OCSP: Connecting AIA responder \"%s\"",
                 sk_OPENSSL_STRING_value(aia, i));
-            status=ocsp_request(c, callback_ctx, cert_id,
+            ocsp_status=ocsp_request(c, callback_ctx, cert_id,
                 sk_OPENSSL_STRING_value(aia, i));
-            if(status!=V_OCSP_CERTSTATUS_UNKNOWN)
+            if(ocsp_status!=V_OCSP_CERTSTATUS_UNKNOWN)
                 break; /* we received a definitive response */
         }
         X509_email_free(aia);
-        if(status!=V_OCSP_CERTSTATUS_GOOD) {
+        if(ocsp_status!=V_OCSP_CERTSTATUS_GOOD) {
             OCSP_CERTID_free(cert_id);
             return 0; /* reject */
         }
     }
 
     OCSP_CERTID_free(cert_id);
+    X509_STORE_CTX_set_error(callback_ctx, saved_error);
     return 1; /* accept */
 }
 
@@ -580,7 +584,8 @@ NOEXPORT int ocsp_check(CLI *c, X509_STORE_CTX *callback_ctx) {
  * V_OCSP_CERTSTATUS_UNKNOWN */
 NOEXPORT int ocsp_request(CLI *c, X509_STORE_CTX *callback_ctx,
         OCSP_CERTID *cert_id, char *url) {
-    int status=V_OCSP_CERTSTATUS_UNKNOWN;
+    int ocsp_status=V_OCSP_CERTSTATUS_UNKNOWN;
+    int response_status;
     int reason;
     int ctx_err=X509_V_ERR_APPLICATION_VERIFICATION;
     OCSP_REQUEST *request=NULL;
@@ -595,23 +600,24 @@ NOEXPORT int ocsp_request(CLI *c, X509_STORE_CTX *callback_ctx,
         sslerror("OCSP: OCSP_REQUEST_new");
         goto cleanup;
     }
-    if(!OCSP_request_add0_id(request, cert_id)) {
+    if(!OCSP_request_add0_id(request, OCSP_CERTID_dup(cert_id))) {
         sslerror("OCSP: OCSP_request_add0_id");
         goto cleanup;
     }
+#if 0
     OCSP_request_add1_nonce(request, NULL, -1);
+#endif
 
     /* send the request and get a response */
     response=ocsp_get_response(c, request, url);
     if(!response)
         goto cleanup;
-    status=OCSP_response_status(response);
-    if(status!=OCSP_RESPONSE_STATUS_SUCCESSFUL) {
+    response_status=OCSP_response_status(response);
+    if(response_status!=OCSP_RESPONSE_STATUS_SUCCESSFUL) {
         s_log(LOG_WARNING, "OCSP: Responder error: %d: %s",
-            status, OCSP_response_status_str(status));
+            response_status, OCSP_response_status_str(response_status));
         goto cleanup;
     }
-    s_log(LOG_DEBUG, "OCSP: Response received");
 
     /* verify the response */
     basic_response=OCSP_response_get1_basic(response);
@@ -619,31 +625,34 @@ NOEXPORT int ocsp_request(CLI *c, X509_STORE_CTX *callback_ctx,
         sslerror("OCSP: OCSP_response_get1_basic");
         goto cleanup;
     }
+#if 0
     if(OCSP_check_nonce(request, basic_response)<=0) {
         s_log(LOG_WARNING, "OCSP: Invalid nonce");
         goto cleanup;
     }
-    if(OCSP_basic_verify(basic_response, NULL,
+#endif
+    if(OCSP_basic_verify(basic_response, X509_STORE_CTX_get_chain(callback_ctx),
             c->opt->revocation_store, c->opt->ocsp_flags)<=0) {
         sslerror("OCSP: OCSP_basic_verify");
         goto cleanup;
     }
-    if(!OCSP_resp_find_status(basic_response, cert_id, &status, &reason,
+    if(!OCSP_resp_find_status(basic_response, cert_id, &ocsp_status, &reason,
             &revoked_at, &this_update, &next_update)) {
         sslerror("OCSP: OCSP_resp_find_status");
         goto cleanup;
     }
-    s_log(LOG_NOTICE, "OCSP: Status: %s", OCSP_cert_status_str(status));
+    s_log(LOG_INFO, "OCSP: Status: %s", OCSP_cert_status_str(ocsp_status));
     log_time(LOG_INFO, "OCSP: This update", this_update);
     log_time(LOG_INFO, "OCSP: Next update", next_update);
     /* check if the response is valid for at least one minute */
     if(!OCSP_check_validity(this_update, next_update, 60, -1)) {
         sslerror("OCSP: OCSP_check_validity");
-        status=V_OCSP_CERTSTATUS_UNKNOWN;
+        ocsp_status=V_OCSP_CERTSTATUS_UNKNOWN;
         goto cleanup;
     }
-    switch(status) {
+    switch(ocsp_status) {
     case V_OCSP_CERTSTATUS_GOOD:
+        s_log(LOG_NOTICE, "OCSP: Certificate accepted");
         break;
     case V_OCSP_CERTSTATUS_REVOKED:
         if(reason==-1)
@@ -664,9 +673,9 @@ cleanup:
         OCSP_RESPONSE_free(response);
     if(basic_response)
         OCSP_BASICRESP_free(basic_response);
-    if(status!=V_OCSP_CERTSTATUS_GOOD)
+    if(ocsp_status!=V_OCSP_CERTSTATUS_GOOD)
         X509_STORE_CTX_set_error(callback_ctx, ctx_err);
-    return status;
+    return ocsp_status;
 }
 
 NOEXPORT OCSP_RESPONSE *ocsp_get_response(CLI *c,
@@ -705,14 +714,20 @@ NOEXPORT OCSP_RESPONSE *ocsp_get_response(CLI *c,
     bio=BIO_new_socket((int)c->fd, BIO_NOCLOSE);
     if(!bio)
         goto cleanup;
-    s_log(LOG_DEBUG, "OCSP: response retrieved");
+    s_log(LOG_DEBUG, "OCSP: Connected %s:%s", host, port);
 
     /* OCSP protocol communication loop */
-    req_ctx=OCSP_sendreq_new(bio, path, req, -1);
+    req_ctx=OCSP_sendreq_new(bio, path, NULL, -1);
     if(!req_ctx) {
         sslerror("OCSP: OCSP_sendreq_new");
         goto cleanup;
     }
+    if(!OCSP_REQ_CTX_add1_header(req_ctx, "Host", host)) {
+        sslerror("OCSP: OCSP_REQ_CTX_add1_header");
+        goto cleanup;
+    }
+    if(!OCSP_REQ_CTX_set1_req(req_ctx, req))
+        goto cleanup;
     while(OCSP_sendreq_nbio(&resp, req_ctx)==-1) {
         s_poll_init(c->fds);
         s_poll_add(c->fds, c->fd, BIO_should_read(bio), BIO_should_write(bio));
@@ -729,7 +744,7 @@ NOEXPORT OCSP_RESPONSE *ocsp_get_response(CLI *c,
 #endif
     /* http://www.mail-archive.com/openssl-users@openssl.org/msg61691.html */
     if(resp) {
-        s_log(LOG_DEBUG, "OCSP: request completed");
+        s_log(LOG_DEBUG, "OCSP: Response received");
     } else {
         if(ERR_peek_error())
             sslerror("OCSP: OCSP_sendreq_nbio");
